from unittest.mock import call, patch

from django.core import mail
from django.urls import reverse
from django.utils.http import urlsafe_base64_encode
from rest_framework import status

from arkindex.project.tests import FixtureAPITestCase


class TestPasswordReset(FixtureAPITestCase):

    def test_password_reset(self):
        resp = self.client.post(
            reverse("api:password-reset"),
            data={"email": self.user.email},
            format="json",
        )
        self.assertEqual(resp.status_code, status.HTTP_201_CREATED)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "Arkindex password reset")
        self.assertListEqual(mail.outbox[0].to, [self.user.email, ])

    def test_password_reset_missing_user(self):
        """
        Test asking for a password reset on an account that does not exist acts like it works
        """
        resp = self.client.post(
            reverse("api:password-reset"),
            data={"email": "nopenopenope@nope.no"},
            format="json",
        )
        self.assertEqual(resp.status_code, status.HTTP_201_CREATED)
        self.assertEqual(len(mail.outbox), 0)

    def test_password_reset_bad_input(self):
        """
        Test asking for a password reset with something else than an email still fails
        """
        resp = self.client.post(
            reverse("api:password-reset"),
            data={"email": "nope"},
            format="json",
        )
        self.assertEqual(resp.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(len(mail.outbox), 0)

    @patch("arkindex.users.serializers.default_token_generator")
    def test_password_reset_confirm(self, token_gen_mock):
        token_gen_mock.check_token.return_value = True
        resp = self.client.post(
            reverse("api:password-reset-confirm"),
            data={
                "uidb64": urlsafe_base64_encode(str(self.user.id).encode()),
                "token": "t0k3n",
                "password": "S3cr37Pa$$w0rd"
            },
            format="json",
        )
        self.assertEqual(resp.status_code, status.HTTP_201_CREATED)
        self.user.refresh_from_db()
        self.assertTrue(self.user.check_password("S3cr37Pa$$w0rd"))
        self.assertEqual(token_gen_mock.check_token.call_count, 1)
        self.assertEqual(token_gen_mock.check_token.call_args, call(self.user, "t0k3n"))

    @patch("arkindex.users.serializers.default_token_generator")
    def test_password_reset_confirm_bad_token(self, token_gen_mock):
        token_gen_mock.check_token.return_value = False
        resp = self.client.post(
            reverse("api:password-reset-confirm"),
            data={
                "uidb64": urlsafe_base64_encode(str(self.user.id).encode()),
                "token": "t0k3n",
                "password": "S3cr37Pa$$w0rd"
            },
            format="json",
        )
        self.assertEqual(resp.status_code, status.HTTP_201_CREATED)
        self.user.refresh_from_db()
        self.assertFalse(self.user.check_password("S3cr37Pa$$w0rd"))
        self.assertEqual(token_gen_mock.check_token.call_count, 1)
        self.assertEqual(token_gen_mock.check_token.call_args, call(self.user, "t0k3n"))

    @patch("arkindex.users.serializers.default_token_generator")
    def test_reset_invalid_password(self, token_gen_mock):
        wrong = {
            "42133742": ["This password is entirely numeric."],
            "user@user.fr": ["The password is too similar to the email address."],
            "hunter2": [
                "This password is too short. It must contain at least 8 characters.",
                "This password is too common."
            ]
        }
        token_gen_mock.check_token.return_value = True
        for pwd, messages in wrong.items():
            response = self.client.post(
                reverse("api:password-reset-confirm"),
                data={"uidb64": urlsafe_base64_encode(str(self.user.id).encode()), "token": "t0k3n", "password": pwd},
                format="json",
            )
            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
            self.assertEqual(response.json(), {"password": messages})
